"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findEntries = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var fast_glob_1 = __importDefault(require("fast-glob"));
var typescript_1 = __importDefault(require("typescript"));
/**
 * main function
 * @param commandName command name
 * @param args arguments
 * @returns exit status
 */
function main(commandName, args) {
    if (args.length !== 2) {
        usage(commandName);
        return 1;
    }
    var srcDirName = args[0], dstDirName = args[1];
    if (!fs_1.default.existsSync(srcDirName)) {
        usage(commandName);
        return 1;
    }
    var srcFileNames = findEntries(srcDirName);
    var program = typescript_1.default.createProgram(srcFileNames, {});
    for (var _i = 0, srcFileNames_1 = srcFileNames; _i < srcFileNames_1.length; _i++) {
        var srcFileName = srcFileNames_1[_i];
        var sourceFile = program.getSourceFile(srcFileName);
        if (sourceFile === undefined) {
            continue;
        }
        var builtCode = build(sourceFile);
        var dstFileName = path_1.default.join(dstDirName, path_1.default.relative(srcDirName, srcFileName));
        output(builtCode, dstFileName);
    }
    return 0;
}
/**
 * show usage
 * @param commandName command name
 */
function usage(commandName) {
    console.log("usage: " + commandName + " SRC_DIR DST_DIR");
}
/**
 * find entries
 * @param dirName directory to search
 * @returns stream of entries
 */
function findEntries(dirName) {
    var source = dirName + "/**/*.{ts,js}";
    return fast_glob_1.default.sync(source);
}
exports.findEntries = findEntries;
/**
 * build source file
 * @param sourceFile source file object
 * @returns built source file
 */
function build(sourceFile) {
    var result = typescript_1.default.transform(sourceFile, [transformerFactory]);
    result.dispose();
    var printer = typescript_1.default.createPrinter();
    return printer.printFile(result.transformed[0]);
}
/**
 * transformer factory
 * @param context transformation context
 * @returns transformer
 */
function transformerFactory(context) {
    return function (rootNode) {
        var sourceFileName = rootNode.getSourceFile().fileName;
        return typescript_1.default.visitNode(rootNode, visitorFactory(sourceFileName, context));
    };
}
/**
 * visitor factory
 * @param sourceFileName source file name
 * @param context transformation context
 * @returns visitor
 */
function visitorFactory(sourceFileName, context) {
    return function (node) {
        if (typescript_1.default.isImportDeclaration(node) || typescript_1.default.isExportDeclaration(node)) {
            // in "import" / "export from" statement
            return typescript_1.default.visitEachChild(node, visitorResolverFactory(sourceFileName), context);
        }
        else {
            return typescript_1.default.visitEachChild(node, visitorFactory(sourceFileName, context), context);
        }
    };
}
/**
 * resolver factory
 * @param sourceFileName source file name
 * @returns visitor
 */
function visitorResolverFactory(sourceFileName) {
    return function (node) {
        if (!typescript_1.default.isStringLiteral(node)) {
            return node;
        }
        // resolve if node is StringLiteral in ImportDeclaration/ExportDeclaration
        var moduleName = node.text;
        var baseDir = path_1.default.dirname(sourceFileName);
        var resolvedModuleName = resolveModuleExtension(moduleName, baseDir);
        return typescript_1.default.createStringLiteral(resolvedModuleName);
    };
}
/**
 * resolves module name
 * @param moduleName modle name
 * @param baseDirName base directory
 * @returns resolved module name
 */
function resolveModuleExtension(moduleName, baseDirName) {
    if (!shouldResolveModuleExtension(moduleName)) {
        return moduleName;
    }
    var resolvedPath = resolveModulePath(moduleName, baseDirName);
    for (var _i = 0, _a = ["", ".ts", ".js"]; _i < _a.length; _i++) {
        var ext = _a[_i];
        var resolvedName = "" + resolvedPath + ext;
        if (fs_1.default.existsSync(resolvedName)) {
            // resolved
            return "" + moduleName + ext;
        }
    }
    // not resolved
    console.warn("[WARN] Module not resolved: " + moduleName);
    return moduleName;
}
/**
 * should resolve?
 * @param moduleName module name
 * @returns Yes/No
 */
function shouldResolveModuleExtension(moduleName) {
    if (path_1.default.isAbsolute(moduleName)) {
        // absolute path
        return true;
    }
    if (moduleName.startsWith(".")) {
        // relative path
        return true;
    }
    return false;
}
/**
 * resolve module path
 * @param moduleName module name to resolve
 * @param baseDir base directory
 * @returns resolved path
 */
function resolveModulePath(moduleName, baseDir) {
    if (path_1.default.isAbsolute(moduleName)) {
        return moduleName;
    }
    else {
        return path_1.default.resolve(baseDir, moduleName);
    }
}
/**
 * output built source
 * @param builtSource built source code
 * @param fileName file name to output
 */
function output(builtSource, fileName) {
    createDirectoriesIfNotExist(fileName);
    fs_1.default.writeFile(fileName, builtSource, function (err) {
        if (err === null) {
            return;
        }
        console.error("[ERROR] Output error: " + err.message);
    });
}
/**
 * create directories recursively if not exist
 * @param fileName file name
 */
function createDirectoriesIfNotExist(fileName) {
    var dirName = path_1.default.dirname(fileName);
    if (fs_1.default.existsSync(dirName)) {
        // already exists
        return;
    }
    fs_1.default.mkdirSync(dirName, {
        recursive: true,
    });
}
main(path_1.default.basename(process.argv[1]), process.argv.slice(2));
